==> hqsoft_core-0.0.18.tar.gz: lib/core_base/localization/global_translations.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:hqsoft_core/core_base/core_base.dart';
import 'package:hqsoft_core/core_base/unit_test/test_asset.dart';

const List<String> _kSupportedLanguages = ["en", "vi"];

class GlobalTranslations {
  Locale _locale;
  Map<dynamic, dynamic> _localizedValues;
  Map<String, String> _cache = {};

  ///
  /// Returns the list of supported locales
  ///
  Iterable<Locale> supportedLocales() =>
      _kSupportedLanguages.map<Locale>((lang) => Locale(lang, ''));

  ///
  /// Return the translation that corresponds to the [key]
  ///
  /// The [key] might be a sequence of [key].[sub-key].[sub-key]
  ///
  /// If [values] is defined, allows replacement of a template value
  ///
  /// If [plural] is defined, looks for the following definitions:
  ///   '=0': if [plural == 0] takes this string
  ///   '=1': if [plural == 1] takes this string
  ///   '>1': if [plural > 1] takes this string
  ///
  /// If [gender] is defined, looks for the following definitions:
  ///   'male': if [gender == GlobalTranslationsGender.male] takes this string
  ///   'female': if [gender == GlobalTranslationsGender.female] takes this string
  ///   'other': if [gender == GlobalTranslationsGender.other] takes this string
  ///
  /// Examples:
  /// text('Value: {{myVal}}, Name: {{myName}}', values: { 'myVal': 123, 'myName': 'Didier'})
  ///
  /// text('demoPage.tests.string_plural', plural: 1)
  ///
  /// text('demoPage.tests.string_gender', gender: GlobalTranslationsGender.male)
  ///
  /// Of course combinations of (plural or gender) and values are possible
  ///
  String text(String key, {Map<String, dynamic> values}) {
    //
    // Processes the template replacements if any
    //
    String _processTemplate(String template) {
      if (values == null) {
        return template;
      }

      String output = template;

      values.forEach((String key, dynamic value) {
        if (value != null && (value is String || value is num)) {
          output = output.replaceAll('{{$key}}', value.toString());
        }
      });

      return output;
    }

    // Return the requested string
    String string = '** $key not found';

    if (_localizedValues != null) {
      // Check if the requested [key] is in the cache
      if (_cache[key] != null) {
        return _processTemplate(_cache[key]);
      }

      // Iterate the key until found or not
      bool found = true;
      Map<dynamic, dynamic> _values = _localizedValues;
      List<String> _keyParts = key.split('.');
      int _keyPartsLen = _keyParts.length;
      int index = 0;
      int lastIndex = _keyPartsLen - 1;

      while (index < _keyPartsLen && found) {
        var value = _values[_keyParts[index]];

        if (value == null) {
          // Not found => STOP
          found = false;
          break;
        }

        // Check if we found the requested key
        if (value is String && index == lastIndex) {
          string = value;

          // Add to cache
          _cache[key] = string;
          break;
        }

        // go to next subKey
        _values = value;
        index++;
      }
    }
    return _processTemplate(string);
  }

  get currentLanguage => _locale == null ? '' : _locale.languageCode;

  Locale get locale => _locale;

  ///
  /// One-time initialization
  ///
  Future<void> init() async {
    if (_locale == null) {
      await setNewLanguage();
    }
    return null;
  }

  Future<bool> update() async {
    var result = await setNewLanguage();
    return result;
  }

  ///
  /// Routine to change the language
  ///
  Future<bool> setNewLanguage([String newLanguage]) async {
    String language = newLanguage;
    if (language == null) {
      language = await languagePreferences.getPreferredLanguage();
    }

    // If not in the preferences, get the current locale
    if (language == '') {
      String currentLocale = Platform.localeName?.toLowerCase();
      if (currentLocale != null) {
        if (currentLocale.length > 2) {
          if (currentLocale[2] == "-" || currentLocale[2] == "_") {
            language = currentLocale.substring(0, 2);
          }
        }
      }
    }
    //iOS : en_Vn/ en_US
    if (!_kSupportedLanguages.contains(language)) {
      language = "";
    }
    print('language: $language');

    // Set the locale
    if (language == "") {
      language = languagePreferences.defaultLanguage;
    }
    _locale = Locale(language, "");

    // Load the language strings
//    String jsonContent = await rootBundle
//        .loadString("assets/locale/locale_${_locale.languageCode}.json");
    String jsonContent = await TestAssetBundle()
        .loadString("assets/locale/locale_${_locale.languageCode}.json");
    _localizedValues = jsonDecode(jsonContent);
    // Clear the cache
    _cache = {};

    return true;
  }

  ///
  /// Singleton Factory
  ///
  static final GlobalTranslations _translations =
      GlobalTranslations._internal();

  factory GlobalTranslations() => _translations;

  GlobalTranslations._internal();
}

GlobalTranslations allTranslations = GlobalTranslations();

==> hqsoft_core-0.0.49.tar.gz: lib/core_base/localization/global_translations.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:hqsoft_core/core_base/core_base.dart';
import 'package:hqsoft_core/core_base/unit_test/test_asset.dart';

const List<String> _kSupportedLanguages = ["en", "vi"];

class GlobalTranslations {
  Locale? _locale;
  Map<dynamic, dynamic>? _localizedValues;
  Map<String, String>? _cache = {};

  ///
  /// Returns the list of supported locales
  ///
  Iterable<Locale> supportedLocales() =>
      _kSupportedLanguages.map<Locale>((lang) => Locale(lang, ''));

  ///
  /// Return the translation that corresponds to the [key]
  ///
  /// The [key] might be a sequence of [key].[sub-key].[sub-key]
  ///
  /// If [values] is defined, allows replacement of a template value
  ///
  /// If [plural] is defined, looks for the following definitions:
  ///   '=0': if [plural == 0] takes this string
  ///   '=1': if [plural == 1] takes this string
  ///   '>1': if [plural > 1] takes this string
  ///
  /// If [gender] is defined, looks for the following definitions:
  ///   'male': if [gender == GlobalTranslationsGender.male] takes this string
  ///   'female': if [gender == GlobalTranslationsGender.female] takes this string
  ///   'other': if [gender == GlobalTranslationsGender.other] takes this string
  ///
  /// Examples:
  /// text('Value: {{myVal}}, Name: {{myName}}', values: { 'myVal': 123, 'myName': 'Didier'})
  ///
  /// text('demoPage.tests.string_plural', plural: 1)
  ///
  /// text('demoPage.tests.string_gender', gender: GlobalTranslationsGender.male)
  ///
  /// Of course combinations of (plural or gender) and values are possible
  ///
  String text(String key, {Map<String, dynamic>? values}) {
    //
    // Processes the template replacements if any
    //
    String _processTemplate(String template) {
      if (values == null) {
        return template;
      }

      String output = template;

      values.forEach((String key, dynamic value) {
        if (value != null && (value is String || value is num)) {
          output = output.replaceAll('{{$key}}', value.toString());
        }
      });

      return output;
    }

    // Return the requested string
    String string = '** $key not found';

    if (_localizedValues != null) {
      // Check if the requested [key] is in the cache
      if (_cache![key] != null) {
        return _processTemplate(_cache![key].toString());
      }

      // Iterate the key until found or not
      bool found = true;
      Map<dynamic, dynamic> _values = _localizedValues!;
      List<String> _keyParts = key.split('.');
      int _keyPartsLen = _keyParts.length;
      int index = 0;
      int lastIndex = _keyPartsLen - 1;

      while (index < _keyPartsLen && found) {
        var value = _values[_keyParts[index]];

        if (value == null) {
          // Not found => STOP
          found = false;
          break;
        }

        // Check if we found the requested key
        if (value is String && index == lastIndex) {
          string = value;

          // Add to cache
          _cache![key] = string;
          break;
        }

        // go to next subKey
        _values = value;
        index++;
      }
    }
    return _processTemplate(string);
  }

  get currentLanguage => _locale == null ? '' : _locale!.languageCode;

  Locale? get locale => _locale;

  ///
  /// One-time initialization
  ///
  Future<void> init() async {
    if (_locale == null) {
      await setNewLanguage();
    }
    return null;
  }

  Future<bool> update() async {
    var result = await setNewLanguage();
    return result;
  }

  ///
  /// Routine to change the language
  ///
  Future<bool> setNewLanguage([String? newLanguage]) async {
    String? language = newLanguage;
    if (language == null) {
      language = await languagePreferences.getPreferredLanguage();
    }

    // If not in the preferences, get the current locale
    if (language == '') {
      String? currentLocale = Platform.localeName.toLowerCase();
      if (currentLocale.length > 2) {
        if (currentLocale[2] == "-" || currentLocale[2] == "_") {
          language = currentLocale.substring(0, 2);
        }
      }
    }
    //iOS : en_Vn/ en_US
    if (!_kSupportedLanguages.contains(language)) {
      language = "";
    }
    print('language: $language');

    // Set the locale
    if (language == "") {
      language = languagePreferences.defaultLanguage;
    }
    _locale = Locale(language, "");

    // Load the language strings
//    String jsonContent = await rootBundle
//        .loadString("assets/locale/locale_${_locale.languageCode}.json");
    String jsonContent = await TestAssetBundle()
        .loadString("assets/locale/locale_${_locale!.languageCode}.json");
    _localizedValues = jsonDecode(jsonContent);
    // Clear the cache
    _cache = {};

    return true;
  }

  ///
  /// Singleton Factory
  ///
  static final GlobalTranslations _translations =
      GlobalTranslations._internal();

  factory GlobalTranslations() => _translations;

  GlobalTranslations._internal();
}

GlobalTranslations allTranslations = GlobalTranslations();
